@@ -307,8 +307,11 @@ sort_info_changed (ETableSortInfo *info, ETable *et)
 	gboolean will_be_grouped = e_table_sort_info_grouping_get_count(info) > 0;
 	if (et->is_grouped || will_be_grouped) {
 		et->need_rebuild = TRUE;
-		if (!et->rebuild_idle_id)
+		if (!et->rebuild_idle_id) {
+			gtk_object_destroy (GTK_OBJECT (et->group));
+			et->group = NULL;
 			et->rebuild_idle_id = g_idle_add_full (20, changed_idle, et, NULL);
+		}
 	}
 }
 
@@ -477,43 +480,28 @@ group_key_press (ETableGroup *etg, int row, int col, GdkEvent *event, ETable *et
 {
 	int return_val = 0;
 	GdkEventKey *key = (GdkEventKey *) event;
-	GdkEventButton click;
+	int y, row_local, col_local;
+	GtkAdjustment *vadj;
+
 	switch (key->keyval) {
 	case GDK_Page_Down:
-		gtk_adjustment_set_value(
-			gtk_layout_get_vadjustment (GTK_LAYOUT (et->table_canvas)),
-			CLAMP(gtk_layout_get_vadjustment (GTK_LAYOUT (et->table_canvas))->value +
-			      (gtk_layout_get_vadjustment (GTK_LAYOUT (et->table_canvas))->page_size - 20),
-			      0,
-			      gtk_layout_get_vadjustment (GTK_LAYOUT (et->table_canvas))->upper -
-			      gtk_layout_get_vadjustment (GTK_LAYOUT (et->table_canvas))->page_size));
-		click.type = GDK_BUTTON_PRESS;
-		click.window = GTK_LAYOUT (et->table_canvas)->bin_window;
-		click.send_event = key->send_event;
-		click.time = key->time;
-		click.x = 30;
-		click.y = gtk_layout_get_vadjustment (GTK_LAYOUT (et->table_canvas))->page_size - 1;
-		click.state = key->state;
-		click.button = 1;
-		gtk_widget_event(GTK_WIDGET(et->table_canvas),
-				 (GdkEvent *) &click);
+		vadj = gtk_layout_get_vadjustment (GTK_LAYOUT (et->table_canvas));
+		y = CLAMP(vadj->value + (2 * vadj->page_size - 50), 0, vadj->upper);
+		y -= vadj->value;
+		e_table_get_cell_at (et, 30, y, &row_local, &col_local);
+		row_local = e_table_view_to_model_row (et, row_local);
+		col_local = e_selection_model_cursor_col (E_SELECTION_MODEL (et->selection));
+		e_selection_model_select_as_key_press (E_SELECTION_MODEL (et->selection), row_local, col_local, key->state);
 		return_val = 1;
 		break;
 	case GDK_Page_Up:
-		gtk_adjustment_set_value(
-			gtk_layout_get_vadjustment (GTK_LAYOUT (et->table_canvas)),
-			gtk_layout_get_vadjustment (GTK_LAYOUT (et->table_canvas))->value -
-			(gtk_layout_get_vadjustment (GTK_LAYOUT (et->table_canvas))->page_size - 20));
-		click.type = GDK_BUTTON_PRESS;
-		click.window = GTK_LAYOUT (et->table_canvas)->bin_window;
-		click.send_event = key->send_event;
-		click.time = key->time;
-		click.x = 30;
-		click.y = 1;
-		click.state = key->state;
-		click.button = 1;
-		gtk_widget_event(GTK_WIDGET(et->table_canvas),
-				 (GdkEvent *) &click);
+		vadj = gtk_layout_get_vadjustment (GTK_LAYOUT (et->table_canvas));
+		y = CLAMP(vadj->value - (vadj->page_size - 50), 0, vadj->upper);
+		y -= vadj->value;
+		e_table_get_cell_at (et, 30, y, &row_local, &col_local);
+		row_local = e_table_view_to_model_row (et, row_local);
+		col_local = e_selection_model_cursor_col (E_SELECTION_MODEL (et->selection));
+		e_selection_model_select_as_key_press (E_SELECTION_MODEL (et->selection), row_local, col_local, key->state);
 		return_val = 1;
 		break;
 	default:
@@ -529,8 +517,11 @@ static void
 et_table_model_changed (ETableModel *model, ETable *et)
 {
 	et->need_rebuild = TRUE;
-	if (!et->rebuild_idle_id)
+	if (!et->rebuild_idle_id) {
+		gtk_object_destroy (GTK_OBJECT (et->group));
+		et->group = NULL;
 		et->rebuild_idle_id = g_idle_add_full (20, changed_idle, et, NULL);
+	}
 }
 
 static void
@@ -560,7 +551,7 @@ et_table_rows_inserted (ETableModel *table_model, int row, int count, ETable *et
 		if (row != row_count - count)
 			e_table_group_increment(et->group, row, count);
 		for (i = 0; i < count; i++)
-			e_table_group_add (et->group, row);
+			e_table_group_add (et->group, row + i);
 		if (et->horizontal_scrolling)
 			e_table_header_update_horizontal(et->header);
 	}
@@ -573,7 +564,7 @@ et_table_rows_deleted (ETableModel *table_model, int row, int count, ETable *et)
 	if (!et->need_rebuild) {
 		int i;
 		for (i = 0; i < count; i++)
-			e_table_group_remove (et->group, row);
+			e_table_group_remove (et->group, row + i);
 		if (row != row_count)
 			e_table_group_decrement(et->group, row, count);
 		if (et->horizontal_scrolling)
@@ -650,7 +641,8 @@ changed_idle (gpointer data)
 	ETable *et = E_TABLE (data);
 
 	if (et->need_rebuild) {
-		gtk_object_destroy (GTK_OBJECT (et->group));
+		if (et->group)
+			gtk_object_destroy (GTK_OBJECT (et->group));
 		et_build_groups(et);
 		gtk_object_set (GTK_OBJECT (et->canvas_vbox),
 				"width", (double) GTK_WIDGET (et->table_canvas)->allocation.width,
@@ -1905,8 +1897,7 @@ et_drag_motion(GtkWidget *widget,
 
 	e_table_get_cell_at (et, x, y, &row, &col);
 
-	if (et->drop_row >= 0 && et->drop_col >= 0 &&
-	    row != et->drop_row && col != et->drop_row) {
+	if (row != et->drop_row && col != et->drop_row) {
 		gtk_signal_emit (GTK_OBJECT (et),
 				 et_signals [TABLE_DRAG_LEAVE],
 				 et->drop_row,
@@ -1916,16 +1907,15 @@ et_drag_motion(GtkWidget *widget,
 	}
 	et->drop_row = row;
 	et->drop_col = col;
-	if (row >= 0 && col >= 0)
-		gtk_signal_emit (GTK_OBJECT (et),
-				 et_signals [TABLE_DRAG_MOTION],
-				 et->drop_row,
-				 et->drop_col,
-				 context,
-				 x,
-				 y,
-				 time,
-				 &ret_val);
+	gtk_signal_emit (GTK_OBJECT (et),
+			 et_signals [TABLE_DRAG_MOTION],
+			 et->drop_row,
+			 et->drop_col,
+			 context,
+			 x,
+			 y,
+			 time,
+			 &ret_val);
 	return ret_val;
 }
 
@@ -1942,37 +1932,34 @@ et_drag_drop(GtkWidget *widget,
 
 	e_table_get_cell_at (et, x, y, &row, &col);
 
-	if (et->drop_row >= 0 && et->drop_col >= 0 &&
-	    row != et->drop_row && col != et->drop_row) {
+	if (row != et->drop_row && col != et->drop_row) {
 		gtk_signal_emit (GTK_OBJECT (et),
 				 et_signals [TABLE_DRAG_LEAVE],
 				 et->drop_row,
 				 et->drop_col,
 				 context,
 				 time);
-		if (row >= 0 && col >= 0)
-			gtk_signal_emit (GTK_OBJECT (et),
-					 et_signals [TABLE_DRAG_MOTION],
-					 row,
-					 col,
-					 context,
-					 x,
-					 y,
-					 time,
-					 &ret_val);
-	}
-	et->drop_row = row;
-	et->drop_col = col;
-	if (row >= 0 && col >= 0)
 		gtk_signal_emit (GTK_OBJECT (et),
-				 et_signals [TABLE_DRAG_DROP],
-				 et->drop_row,
-				 et->drop_col,
+				 et_signals [TABLE_DRAG_MOTION],
+				 row,
+				 col,
 				 context,
 				 x,
 				 y,
 				 time,
 				 &ret_val);
+	}
+	et->drop_row = row;
+	et->drop_col = col;
+	gtk_signal_emit (GTK_OBJECT (et),
+			 et_signals [TABLE_DRAG_DROP],
+			 et->drop_row,
+			 et->drop_col,
+			 context,
+			 x,
+			 y,
+			 time,
+			 &ret_val);
 	et->drop_row = -1;
 	et->drop_col = -1;
 	return ret_val;
