@@ -342,7 +342,9 @@ add_single_category (int model_row, gpointer closure)
 	ECategories *categories = closure;
 	if (model_row >= e_categories_master_list_count (categories->priv->ecml)) {
 		e_categories_master_list_append (categories->priv->ecml,
-						 categories->priv->category_list[model_row]);
+						 categories->priv->category_list[model_row],
+						 NULL,  /* FIXME: color for this category */
+						 NULL); /* FIXME: icon for this category */
 	}
 }
 
